//
//  UpnpAvContentDirectoryService.m
//  uPnp2
//
//  Created by rk on 10/03/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "UpnpAvContentDirectoryService.h"
#import "OrderedDictionary.h"
#import "uPnpAppDelegate.h"


@implementation UpnpAvContentDirectoryService

#pragma mark Designated Initializer
-(id)initWithService:(UpnpAvService *)theService {
	self = [super init];
	service = theService;
	return self;
}

#pragma mark AsynchUpnpCommandDelegate methods
- (void)handleAsynchUpnpCommandResult:(GDataXMLDocument *)parsedResult {
    GDataXMLElement *resultNode = nil;
    NSError *error;
    NSArray *resultNodes = [parsedResult nodesForXPath:@"//soap:Envelope/soap:Body/upnpCD:BrowseResponse/Result" 
                                            namespaces:[UpnpAvService namespaceDict] 
                                                 error:&error];
    if (resultNodes && (resultNodes.count > 0)) {
        resultNode = (GDataXMLElement *)[resultNodes objectAtIndex:0];
    }
    if (resultNode) {
		NSLog(@"%@: Result: %@", self, resultNode.stringValue);
		// resultNode.stringValue contains (escaped) XML to be parsed.
        GDataXMLDocument *didlLiteNodeDoc = nil;
        didlLiteNodeDoc = [[GDataXMLDocument alloc] initWithXMLString:resultNode.stringValue
                                                                                options:0 
                                                                                  error:&error]; 
		NSLog(@"%@: didlLiteNodeDoc:%@", self, didlLiteNodeDoc);
        NSArray *containers = [didlLiteNodeDoc nodesForXPath:@"//didlLite:DIDL-Lite/didlLite:container" 
                                                       namespaces:[UpnpAvService namespaceDict]                                                     error:&error];
        NSArray *items = [didlLiteNodeDoc nodesForXPath:@"//didlLite:DIDL-Lite/didlLite:item" 
                                             namespaces:[UpnpAvService namespaceDict]  
                                                  error:&error];
		NSLog(@"%@: resultNode.children: %@, containers.count:%d, items.count:%d", 
			  self, 
			  resultNode.children, 
			  containers.count, 
			  items.count);
		
		if ([getContainerContentDelegate respondsToSelector:@selector(getContainerContentHandler::)]) {
			[getContainerContentDelegate getContainerContentHandler:containers :items];
		}
        //[didlLiteNodeDoc release];
	
	}
}

#pragma mark -
#pragma mark unsynthesized Properties

- (UpnpHttpServer *)httpServer {
    uPnpAppDelegate *appDelegate = (uPnpAppDelegate *)[[UIApplication sharedApplication] delegate];
    UpnpHttpServer *httpServer = appDelegate.httpServer;
    return httpServer;
}

#pragma mark -

-(void)getContainerContent:(NSString *)containerId: (id)delegate {
	getContainerContentDelegate = delegate;
	[self doGetContainerContentCommand:containerId];
}

-(void)fetchResource:(NSString *)resourceUri {
    NSString *destUri = [NSString stringWithFormat:@"%@/a.jpg", self.httpServer.baseUrl]; 
    [self doExportResourceCommand:resourceUri :destUri];    
}


-(void)doGetContainerContentCommand:(NSString *)containerId {
	NSString *commandName = @"Browse";
	OrderedDictionary *args = [[OrderedDictionary alloc] init]; // Get 402 Upnp error if params are sent in wrong order...
	[args setValue:containerId forKey:@"ObjectID"];
	[args setValue:@"BrowseDirectChildren" forKey:@"BrowseFlag"];
	[args setValue:@"*" forKey:@"Filter"];
	[args setValue:@"0" forKey:@"StartingIndex"];
	[args setValue:@"0" forKey:@"RequestedCount"];
	[args setValue:@"" forKey:@"SortCriteria"];
	
	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = self;		
	
	[service executeCommandAsync:command];
}

-(void)doExportResourceCommand:(NSString *)sourceUri : (NSString *)destUri{
    NSString *commandName = @"ExportResource";
    OrderedDictionary *args = [[OrderedDictionary alloc] init];
    [args setValue:sourceUri forKey:@"SourceURI"];
    [args setValue:destUri forKey:@"DestinationURI"];
    
    AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = self;		
	
	[service executeCommandAsync:command];
}


@end
